#! /bin/sh
#----------------------------------------------------------------------------------------
# /tmp/install.sh - install script for smartmon
#
# Copyright (c) 2004-2008 Jens Berger <jberger@gmx.net>
#               2021-2024 Marcus Roeckrath <marcus.roeckrath@gmx.de>
#
# Creation:     2004-09-02 jb
# Last Update:  2024-03-13 Marcus Roeckrath
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# including eisfair specific files
. /var/install/include/eislib
. /var/install/include/check-eisfair-version

installfile=/etc/config.d/smartmon

# delete very old logs
rm -f /var/log/smart_*.log

mkdir -p /var/smartmon/log
mkdir -p /var/smartmon/plot
mkdir -p /etc/smartd_warning.d
chmod 0755 /etc/smartd_warning.d

if [ -f /var/install/packages/apache ]
then
    . /etc/config.d/apache
    if [ -d $APACHE_DOCUMENT_ROOT ]
    then
        rm -f $APACHE_DOCUMENT_ROOT/smartmon && ln -sf /var/smartmon/plot $APACHE_DOCUMENT_ROOT/smartmon
    fi
elif [ -f /var/install/packages/apache2 ]
then
    . /etc/config.d/apache2
    if [ -d $APACHE2_DOCUMENT_ROOT ]
    then
        rm -f $APACHE2_DOCUMENT_ROOT/smartmon && ln -sf /var/smartmon/plot $APACHE2_DOCUMENT_ROOT/smartmon
    fi
fi

/var/install/bin/add-menu "setup.services.menu" "setup.services.smartmon.menu" "S.M.A.R.T. Monitoring Services"

# remove symlink
rm -f /usr/local/htdocs/smartmon
if [ -f /usr/local/bin/httpd-menu.sh ]
then
    /usr/local/bin/httpd-menu.sh rem "./smartmon/index.html"
    ln -sf /var/smartmon/plot /usr/local/htdocs/smartmon
    /usr/local/bin/httpd-menu.sh add "./smartmon/index.html" "S.M.A.R.T." "Tools"
fi

if [ -f $installfile ]
then
    # previous version found - upgrading
    update=true
else
    # new installation - parameter check
    cp /etc/default.d/smartmon $installfile
fi

/usr/sbin/service daemon-reload

/var/install/config.d/smartmon-update.sh update

if ${update:-false}
then
    /var/install/config.d/smartmon.sh >/dev/null 2>&1
fi

# Update drivedb.h
if [ ! -x "/usr/bin/gpg" ] ; then
    mecho --info "If download of drives database should be verified install gnupg2 package."
fi
/var/install/bin/smartmon-drivedb-update

exit 0
