#!/bin/sh
#----------------------------------------------------------------------------
# smartmon-update.sh - update or generate new smartmon configuration
#
# Copyright (c) 2004-2008 Jens Berger <jberger@gmx.net>
#               2017-2023 Marcus Roeckrath <marcus.roeckrath@gmx.de>
#
# Creation:     2004-09-02 jb
# Last Update:  2023-09-28 mr
#
# Parameters:
#
#     smartmon-update.sh [update]   - check/update configuration file
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# reading eislib 
. /var/install/include/eislib

# reading configlib
. /var/install/include/configlib

smartmonfile=/etc/config.d/smartmon
generate_conf=$smartmonfile

sepline ()
{
echo '#------------------------------------------------------------------------------'
}

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    renamed=0

    if [ $renamed = 1 ]; then
        mecho --info "Read documentation for renamed parameter(s)!"
    fi
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------

modify_variables ()
{
    modified=0

    if [ $SMARTMON_DEVICE_N -eq 0 ]; then
        IMAX=1
    else
        IMAX=$SMARTMON_DEVICE_N
    fi

    for IDX in `/usr/bin/seq 1 $IMAX`; do
        eval smartmon_device_type='$SMARTMON_DEVICE_'$IDX'_TYPE'
        # -------------------------------------------------------
        if [ $smartmon_device_type = "ide" ]; then
            mecho --info "Changed: SMARTMON_DEVICE_"$IDX"_TYPE='ide' to"
            mecho --info "         SMARTMON_DEVICE_"$IDX"_TYPE='ata'"
            eval "SMARTMON_DEVICE_"$IDX"_TYPE='ata'"
            modified=1
        fi
        # -------------------------------------------------------
    done

    if [ $modified = 1 ]; then
        mecho --info "Read documentation for modified parameter(s)!"
    fi
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    added=0

    if [ $SMARTMON_DEVICE_N -eq 0 ]; then
        IMAX=1
    else
        IMAX=$SMARTMON_DEVICE_N
    fi

    for IDX in `/usr/bin/seq 1 $IMAX`; do
        # -------------------------------------------------------
        if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_ACTIVE' $source_conf`" ]; then
            mecho --info "Added: SMARTMON_DEVICE_"$IDX"_ACTIVE='yes'"
            eval "SMARTMON_DEVICE_"$IDX"_ACTIVE='yes'"
            added=1
        fi
        # -------------------------------------------------------
        if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_PLOT' $source_conf`" ]; then
            mecho --info "Added: SMARTMON_DEVICE_"$IDX"_PLOT='yes'"
            eval "SMARTMON_DEVICE_"$IDX"_PLOT='yes'"
            added=1
        fi
        # -------------------------------------------------------
        if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_WARN_N' $source_conf`" ]; then
            mecho --info "Added: SMARTMON_DEVICE_"$IDX"_WARN_N='1'"
            eval "SMARTMON_DEVICE_"$IDX"_WARN_N='1'"
            added=1
        fi
        # -------------------------------------------------------
        eval smartmon_device_warn='$SMARTMON_DEVICE_'$IDX'_WARN_N'
        # if [ "$smartmon_device_warn" = "0" ]; then
        #     JMAX=1
        # else
            JMAX=$smartmon_device_warn
        # fi
        for JDX in `/usr/bin/seq 1 $JMAX`; do
            # -------------------------------------------------------
            if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_WARN_'$JDX'_ATTRIBUTE' $source_conf`" ]; then
                mecho --info "Added: SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_ATTRIBUTE='Temperature_Celsius'"
                eval "SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_ATTRIBUTE='Temperature_Celsius'"
                added=1
            fi
            # -------------------------------------------------------
            if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_WARN_'$JDX'_KIND' $source_conf`" ]; then
                mecho --info "Added: SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_KIND='raw'"
                eval "SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_KIND='raw'"
                added=1
            fi
            # -------------------------------------------------------
            if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_WARN_'$JDX'_RELATION' $source_conf`" ]; then
                mecho --info "Added: SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_RELATION='over'"
                eval "SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_RELATION='over'"
                added=1
            fi
            # -------------------------------------------------------
            if [ -z "`grep '^SMARTMON_DEVICE_'$IDX'_WARN_'$JDX'_THRESHOLD' $source_conf`" ]; then
                mecho --info "Added: SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_THRESHOLD='30'"
                eval "SMARTMON_DEVICE_"$IDX"_WARN_"$JDX"_THRESHOLD='30'"
                added=1
            fi
            # -------------------------------------------------------
        done
        # -------------------------------------------------------
    done
    # -------------------------------------------------------
    if [ -z "`grep '^SMARTMON_PLOT' $source_conf`" ]; then
        mecho --info "Added: SMARTMON_PLOT='yes'"
        eval "SMARTMON_PLOT='yes'"
        added=1
    fi
    # -------------------------------------------------------
    if [ -z "`grep '^SMARTMON_PLOT_CRON_SCHEDULE' $source_conf`" ]; then
        mecho --info "Added: SMARTMON_PLOT_CRON_SCHEDULE='05 */1 * * *'"
        eval "SMARTMON_PLOT_CRON_SCHEDULE='05 */1 * * *'"
        added=1
    fi
    # -------------------------------------------------------
    if [ -z "`grep '^SMARTMON_PLOT_SHOW_SN' $source_conf`" ]; then
        mecho --info "Added: SMARTMON_PLOT_SHOW_SN='yes'"
        eval "SMARTMON_PLOT_SHOW_SN='yes'"
        added=1
    fi

    if [ $added -eq 1 ]; then
        mecho --info "Read documentation for new parameter(s)!"
    fi
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    deleted=0

    for varname in "SMARTMON_GENSTATS SMARTMON_GENSTATS_CRON_SCHEDULE"; do
        if [ ! -z "`grep \"^$varname\" $source_conf`" ]; then
            mecho --info "Deleted: $varname"
            deleted=1
        fi
    done

    if [ $SMARTMON_DEVICE_N -eq 0 ]; then
        IMAX=1
    else
        IMAX=$SMARTMON_DEVICE_N
    fi

    for IDX in `/usr/bin/seq 1 $IMAX`; do
        # -------------------------------------------------------
        if [ ! -z "`grep ^SMARTMON_DEVICE_${IDX}_3WARE_DRIVES $source_conf`" ]; then
            mecho --info "Deleted:  SMARTMON_DEVICE_"$IDX"_3WARE_DRIVES"
            deleted=1
        fi
        # -------------------------------------------------------
    done

}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    {   echo '#------------------------------------------------------------------------------'
        echo '# /etc/config.d/smartmon - configuration for S.M.A.R.T. services on EIS/FAIR'
        echo '#'
        echo '# Copyright (c) 2004-2008 Jens Berger <jberger@gmx.net>'
        echo '#               2017 Marcus Roeckrath <marcus.roeckrath@gmx.de>'
        echo '#'
        echo '# Creation:     09.02.2004 jb'
        echo '# Last Update:  $Id$'
        echo '#'
        echo '#'
        echo '# This program is free software; you can redistribute it and/or modify'
        echo '# it under the terms of the GNU General Public License as published by'
        echo '# the Free Software Foundation; either version 2 of the License, or'
        echo '# (at your option) any later version.'
        echo '#------------------------------------------------------------------------------'
        echo
        sepline
        echo "# General settings"
        sepline

        printvar 'START_SMARTMON' "start smartmon: yes or no"

        echo
        printvar 'SMARTMON_DEVICE_N' "smartmon for N filesystems"

        echo
        sepline
        echo "# Device specific settings"
        sepline

        if [ $SMARTMON_DEVICE_N -eq 0 ]; then
            IMAX=1
        else
            IMAX=$SMARTMON_DEVICE_N
        fi

        for IDX in `/usr/bin/seq 1 $IMAX`; do
            printvar 'SMARTMON_DEVICE_'$IDX'_ACTIVE' "yes or no"
            echo
            printvar 'SMARTMON_DEVICE_'$IDX'_TYPE' "device type, read smartctl man page"
            echo
            printvar 'SMARTMON_DEVICE_'$IDX'_NAME' "device name, e.g /dev/sda for ata"
            echo
            printvar 'SMARTMON_DEVICE_'$IDX'_ARGS' "additional arguments for finetuning"
            printvar ''                            "or problematic drives"
            echo
            printvar 'SMARTMON_DEVICE_'$IDX'_OVERRIDE_DEFAULT' "override default smartmon"
            printvar ''                                        "setting? yes or no"
            echo
            printvar 'SMARTMON_DEVICE_'$IDX'_PLOT' "should smartmon log and plot history"
            printvar ''                            "of smart values for this device?"
            echo
            printvar 'SMARTMON_DEVICE_'$IDX'_WARN_N' "number of attributes to watch"
            eval smartmon_device_warn='$SMARTMON_DEVICE_'$IDX'_WARN_N'

            # if [ "$smartmon_device_warn" = "0" ]; then
            #     IDXX_MAX=1
            # else
                IDXX_MAX=$smartmon_device_warn
            # fi
            for IDXX in `/usr/bin/seq 1 $IDXX_MAX`; do
                echo
                printvar 'SMARTMON_DEVICE_'$IDX'_WARN_'$IDXX'_ATTRIBUTE' "watch for attribute"
                echo
                printvar 'SMARTMON_DEVICE_'$IDX'_WARN_'$IDXX'_KIND' "watch 'raw' or 'smart' values?"
                echo
                printvar 'SMARTMON_DEVICE_'$IDX'_WARN_'$IDXX'_RELATION' "warn 'over' or 'under' threshold?"
                echo
                printvar 'SMARTMON_DEVICE_'$IDX'_WARN_'$IDXX'_THRESHOLD' "threshold value"
            done
        done

        echo
        sepline
        echo "# Mail settings"
        sepline
        printvar 'SMARTMON_STARTUP_TESTMAIL' "send testmail when smartd"
        printvar ''                          "is starting? yes or no"
        echo
        printvar 'SMARTMON_FAILURE_MAIL_TO' "recipient of failure mails"
        echo
        sepline
        echo "# Plot settings"
        sepline
        printvar 'SMARTMON_PLOT' "log and plot? yes or no"
        echo
        printvar 'SMARTMON_PLOT_SHOW_SN' "show drive serial number on plot?"
        echo
        sepline
        echo "# Cron settings"
        sepline
        printvar 'SMARTMON_PLOT_CRON_SCHEDULE' "time to log and plot"
        sepline
        echo "# End"
        sepline
    } > $generate_conf
}

source_conf=$smartmonfile

case "$1"
in
    update)
        # update configuration
        if [ -f $source_conf ]; then
            # previous configuration file exists
            . $source_conf
            rename_variables
            modify_variables
            add_variables
            delete_variables
            create_config
        else
            mecho --error "No configuration $source_conf found - exiting."
        fi
    ;;
    *)
        mecho
        mecho "Use one of the following options:"
        mecho
        mecho "  smartmon-update.sh [update] - the file $installfile will be read, the configuration will"
        mecho "                            be checked and an updated smartmon configuration file will be written."
        mecho
    ;;
esac
