#!/bin/sh
# ----------------------------------------------------------------------------
# /var/install/dialog.d/SMARTMON_DEVICE_TYPE.sh - script dialog for ece
#
# Creation:     2023-09-27 mr
# Last update:  2023-10-01 mr
#
# Copyright (c) 2023 Marcus Roeckrath, marcus(dot)roeckrath(at)gmx(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

. /var/install/include/cuilib
. /var/install/include/ecelib

#----------------------------------------------------------------------------
# control constants
#----------------------------------------------------------------------------

IDC_CHECK='1'
IDC_LISTVIEW='20'
IDC_EDIT_PATTERN='11'
IDC_BUTOK='100'
IDC_BUTCANCEL='100'

#----------------------------------------------------------------------------
# global variables
#----------------------------------------------------------------------------
valueBackup=''

#----------------------------------------------------------------------------
# is_pattern
#    check if parameter is not empty
#----------------------------------------------------------------------------
is_pattern()
{
    echo "${1}" | grep -E '.+'
}

#----------------------------------------------------------------------------
# cancel_button_clicked
#         $p2 --> dialog window handle
#         $p3 --> control's window handle
#----------------------------------------------------------------------------
cancel_button_clicked()
{
    # use the backup
    value=${valueBackup}
    valueBackup=''

    cui_window_close "${p2}" "${IDCANCEL}"
    cui_return 1
}

#----------------------------------------------------------------------------
# ok_button_clicked
#         $p2 --> dialog window handle
#         $p3 --> control's window handle
#----------------------------------------------------------------------------
ok_button_clicked()
{
    local dlg="${p2}"
    local check=''
    local lb=''
    local can_close=0

    # proof if listview or pattern
    cui_window_getctrl ${dlg} ${IDC_CHECK}
    check="${p2}"
    cui_checkbox_getcheck ${check}
    val="${p2}"

    if [ ${val} = 1 ]
    then
        # get pattern
        cui_window_getctrl ${dlg} ${IDC_EDIT_PATTERN}
        check="${p2}"
        cui_edit_gettext "${check}"
        if is_pattern "${p2}"
        then
            can_close=1
        else
            cui_message ${dlg} "Value can not be empty!" "Error" "$MB_ERROR}"
        fi
   else
       # get value fom listview
        cui_window_getctrl ${dlg} ${IDC_LISTVIEW}
        lb="${p2}"
        # get index
        cui_listview_getsel "${lb}"
        index="${p2}"
        cui_listview_gettext "${lb}" "${index}" "0"
        can_close=1
    fi

    # return value
    value="${p2}"
    if [ ${can_close} = 1 ]
    then
        cui_window_close "${dlg}" "${IDOK}"
    fi

    cui_return 1
}

#----------------------------------------------------------------------------
# checkbox_clicked
#         $p2 --> dialog window handle
#         $p3 --> control's window handle
#----------------------------------------------------------------------------
checkbox_clicked()
{
    local dlg="${p2}"
    local chb="${p3}"

    cui_checkbox_getcheck ${chb}
    local val="${p2}"

    if [ ${val} = 1 ]
    then
        # hide listview
        cui_window_getctrl ${dlg} ${IDC_LISTVIEW}
        cui_window_hide    ${p2}  1
        # enable pattern
        cui_window_getctrl ${dlg} ${IDC_EDIT_PATTERN}
        cui_window_hide    ${p2}  0
        cui_window_setfocus ${p2}
    else
        # unhide listview
        cui_window_getctrl ${dlg} ${IDC_LISTVIEW}
        cui_window_hide    ${p2}  0
        # disable pattern
        cui_window_getctrl ${dlg} ${IDC_EDIT_PATTERN}
        cui_window_hide    ${p2}  1
    fi

    cui_return 1
}

#----------------------------------------------------------------------------
# listview_changed
#         $p2 --> dialog window handle
#         $p3 --> control's window handle
#----------------------------------------------------------------------------
listview_changed()
{
    cui_return 1
}

#----------------------------------------------------------------------------
# testdlg_create_hook
#         $p2 --> dialog window handle
#----------------------------------------------------------------------------
testdlg_create_hook()
{
    valueBackup=${value}
    local dlg="${p2}"
    local ctrl

    local count="0"
    local idx
    local selection=""

    local maxwidth1="0"
    local maxwidth2="0"

    local setStyle=${CWS_HIDDEN}
    local setChecked=0
    local setValue=''
    if is_pattern "${value}"
    then
       setStyle=${CWS_NONE}
       setChecked=1
       setValue="${value}"
    fi

    # create checkbox
    if cui_checkbox_new "${dlg}" "Value" 0 1 10 1 ${IDC_CHECK} ${CWS_NONE} ${CWS_NONE}
    then
        ctrl="${p2}"
        cui_checkbox_callback "${ctrl}" "${CHECKBOX_CLICKED}" "${dlg}" checkbox_clicked
        cui_checkbox_setcheck "${ctrl}" ${setChecked}
        cui_window_create     "${ctrl}"
    fi

    # create edit
    if cui_edit_new "${dlg}" "" 11 1 67 1 40 ${IDC_EDIT_PATTERN} ${setStyle} ${CWS_NONE}
    then
        ctrl="${p2}"
        cui_window_create     "${ctrl}"
        cui_edit_settext      "${ctrl}" "${setValue}"
    fi

    # create listview
    if cui_listview_new "${dlg}" "" 0 3 78 18 2 ${IDC_LISTVIEW} ${CWS_NONE} ${CWS_BORDER}
    then
        ctrl="${p2}"
        cui_listview_callback  "${ctrl}" "${LISTVIEW_CHANGED}" "${dlg}" listview_changed
        cui_window_setcolors  "${ctrl}" "MENU"
        cui_window_create     "${ctrl}"
        cui_listview_setcoltext  "${ctrl}" "0" "Device type "
        cui_listview_setcoltext  "${ctrl}" "1" "Description"

        cui_listview_settitlealignment  "${ctrl}" "0" "${ALIGN_LEFT}"
        cui_listview_settitlealignment  "${ctrl}" "1" "${ALIGN_LEFT}"

        # transfer entries into the listview control
        LOCAL_OLD_IFS="${IFS}"
        IFS=","
        for sel in ${sellist}
        do
            local column1=${sel%|*}
            local column2=${sel#*|}

            [ "${#column1}" -gt "${maxwidth1}" ] && maxwidth1="${#column1}"
            [ "${#column2}" -gt "${maxwidth2}" ] && maxwidth2="${#column2}"

            cui_listview_add     "${ctrl}" && idx="${p2}"
            cui_listview_settext "${ctrl}" "${idx}" "0" "${column1}"
            cui_listview_settext "${ctrl}" "${idx}" "1" "${column2}"

            if [ "${column1}" = "${value}" ]
            then
                selection="${idx}"
            fi
            count=$((${count} + 1))
        done
        IFS="${LOCAL_OLD_IFS}"

        # set value or hide listview
        if [ ${setChecked} = 0 ]
        then
            cui_listview_setsel "${ctrl}" "${selection}"
        else
            cui_window_hide    "${ctrl}" 1
        fi
    fi

    # create OK button
    if cui_button_new "${dlg}" "&OK" 27 22 10 1 ${IDC_BUTOK} ${CWS_DEFOK} ${CWS_NONE}
    then
        ctrl="${p2}"
        cui_button_callback   "${ctrl}" "${BUTTON_CLICKED}" "${dlg}" ok_button_clicked
        cui_window_create     "${ctrl}"
    fi

    # create CANCEL button
    if cui_button_new "${dlg}" "&Cancel" 38 22 10 1 ${IDC_BUTCANCEL} ${CWS_DEFCANCEL} ${CWS_NONE}
    then
        ctrl="${p2}"
        cui_button_callback   "${ctrl}" "${BUTTON_CLICKED}" "${dlg}"  cancel_button_clicked
        cui_window_create     "${ctrl}"
    fi

    cui_return 1
}

# ----------------------------------------------------------------------------
# exec_dailog
# ece --> request to create and execute dialog
#         $p2 --> main window handle
#         $p3 --> name of config variable
# ----------------------------------------------------------------------------
exec_dialog()
{
    local win="${p2}"
    local res="${IDCANCEL}"

    # create window
    if cui_window_new "${p2}" 0 0 80 25 $((${CWS_POPUP} + ${CWS_CENTERED} + ${CWS_BORDER}))
    then
        local dlgwin="${p2}"
        cui_window_setcolors   "${dlgwin}" "DIALOG"
        cui_window_settext     "${dlgwin}" "Select or enter a disk drive"
        cui_window_sethook     "${dlgwin}" "${HOOK_CREATE}"  testdlg_create_hook
        cui_window_create      "${dlgwin}"

        cui_window_modal       "${dlgwin}"
        res="${p2}"
        cui_window_destroy     "${dlgwin}"
    fi

    cui_return "${res}"

}

# ----------------------------------------------------------------------------
# Create the selection list out of all configured and active filesets
# ----------------------------------------------------------------------------
create_selection()
{
    sellist="auto|Trying to guess disk type"
    sellist="${sellist}"",ata|SATA/ATA disks"
    sellist="${sellist}"",scsi|SCSI disks"
    sellist="${sellist}"",nvme|NVM Express disks"
    sellist="${sellist}"",sat|ATA disks with SCSI to ATA Translation Layer"
    sellist="${sellist}"",usbcypress|ATA disks behind a Cypress USB to PATA bridge"
    sellist="${sellist}"",usbjmicron|SATA disks behind a JMicron USB to PATA/SATA bridge"
    sellist="${sellist}"",usbprolific|SATA disks behind a Profilic USB to SATA bridge"
    sellist="${sellist}"",usbsunplus|SATA disks behind a SunplusIT USB to SATA bridge"
    sellist="${sellist}"",sntjmicron|NVMe disks behind a JMicron USB to NVMe bridge"
    sellist="${sellist}"",sntrealtek|NVMe disks behind a Realtek USB to NVMe bridge"
    sellist="${sellist}"",marvell|SATA disks behind a Marvell chip-set controller (deprecated)"
    sellist="${sellist}"",3ware|ATA disks connected to a 3ware RAID controller"
    sellist="${sellist}"",cciss|SCSI/SAS disk connected to a cciss RAID controller"
}

# ----------------------------------------------------------------------------
# main routine
# ----------------------------------------------------------------------------

sellist=''

create_selection

cui_init
cui_run

# ----------------------------------------------------------------------------
# end
# ----------------------------------------------------------------------------

exit 0
